/*
 * This source file is part of ImageViewer
 * For the latest info, see http://thierry.vouriot.free.fr
 * E-mail : vouriott@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 */
#include "imageviewer.h"

#include <QHeaderView.h>
#include <QDirModel.h>
#include <QFileDialog.h>
#include <QMessageBox.h>
#include <QEvent.h>
#include <QSize.h>
#include <QImage.h>
#include <QPixmap.h>
#include <QScrollBar.h>

//=======================================================================
// ImageViewer implementation
//=======================================================================

//-----------------------------------------------------------------------
ImageViewer::ImageViewer(QWidget* pParent)
    : QScrollArea(pParent)
{
    m_pImageLabel = new QLabel();
    m_pImageLabel->setBackgroundRole(QPalette::Shadow);

    m_pImageLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
    m_pImageLabel->setScaledContents(true);

    this->setBackgroundRole(QPalette::Shadow);
    this->setWidget(m_pImageLabel);

    m_dScaleFactor = 1.0;
    m_strImagePath = "";
}

//-----------------------------------------------------------------------
void ImageViewer::open()
{
    QString strFilePath = QFileDialog::getOpenFileName(this,
                                    tr("Open File"), QDir::currentPath());
    if (!strFilePath.isEmpty()) {

        openFile(strFilePath);
    }
}

//-----------------------------------------------------------------------
void ImageViewer::openFile(QString strFilePath)
{
    QImage Image(strFilePath);

    if (Image.isNull()) {
        QMessageBox::information(this, tr("Image Viewer"),
                                 tr("Cannot load %1.").arg(strFilePath));
        return;
    }
    m_pImageLabel->setPixmap(QPixmap::fromImage(Image));
    m_dScaleFactor = 1.0;
    m_strImagePath = strFilePath;
    m_pImageLabel->adjustSize();

    emit imgOpened(strFilePath);

    centerImage();
}

//-----------------------------------------------------------------------
void ImageViewer::resizeEvent(QResizeEvent* pEvent)
{
    if (m_pImageLabel->pixmap())
        centerImage();

     QScrollArea::resizeEvent(pEvent);
}

//-----------------------------------------------------------------------
void ImageViewer::centerImage()
{

    // We compute the margins to center the image
    const QPixmap* pPixmap = m_pImageLabel->pixmap();
    int nImgW = (int)(pPixmap->width()*m_dScaleFactor);
    int nImgH = (int)(pPixmap->height()*m_dScaleFactor);
    int nMarginH = (viewport()->width() - nImgW);
    if (nMarginH < 0) nMarginH = 0;
    int nMarginV = (viewport()->height() - nImgH);
    if (nMarginV < 0) nMarginV = 0;
    m_pImageLabel->setGeometry(0,0,nImgW + nMarginH, nImgH + nMarginV);
    m_pImageLabel->setContentsMargins(nMarginH/2,nMarginV/2,nMarginH/2,nMarginV/2);
}

//-----------------------------------------------------------------------
void ImageViewer::scaleImage(double dScale)
{
    m_dScaleFactor = dScale;
    centerImage();
    emit zoomed(m_dScaleFactor);
}

//-----------------------------------------------------------------------
void ImageViewer::zoomIn()
{
    scaleImage(m_dScaleFactor * 1.25);
}

//-----------------------------------------------------------------------
void ImageViewer::zoomOut()
{
    scaleImage(m_dScaleFactor * 0.8);
}

//-----------------------------------------------------------------------
void ImageViewer::normalSize()
{
    scaleImage(1.0);
}

//-----------------------------------------------------------------------
void ImageViewer::contextMenuEvent (QContextMenuEvent* pEvent)
{
    m_pPopupMenu->move(pEvent->globalX(), pEvent->globalY());
    m_pPopupMenu->show();
    pEvent->accept();
}







