/*
 * This source file is part of ImageViewer
 * For the latest info, see http://thierry.vouriot.free.fr
 * E-mail : vouriott@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 */

#ifndef _IMAGEVIEWER_H_
#define _IMAGEVIEWER_H_

#include <QScrollArea.h>
#include <QLabel.h>
#include <QMenu.h>

//-----------------------------------------------------------------------
/** Image viewer.
 */
class ImageViewer : public QScrollArea
{
    Q_OBJECT

public:

    /** Constructor.
     *  @param pParent Parent widget.
     */
    ImageViewer(QWidget* pParent = 0);

    /** Change the PopupMenu.
     *  @param pPopupMenu The new PopupMenu.
     */
    void setPopupMenu(QMenu* pPopupMenu)
    {
        m_pPopupMenu = pPopupMenu;
    }

    /** Return the path of displayed image.
     *  @return The image path.
     */
    const QString& getImagePath()
    {
        return m_strImagePath;
    }

public slots:

    /** Open a file using QFileDialog.
     */
    void open();

    /** Open a file (image).
     *  @param strFilePath Path of the file.
     */
    void openFile(QString strFilePath);

    /** Zoom (in) on the displayed image.
     */
    void zoomIn();

    /** Zoom (out) on the displayed image.
     */
    void zoomOut();

    /** Display the image in the normal scale.
     */
    void normalSize();

signals:

    /** Signal for a new image opening.
     *  @param strImgPath Path of the new image.
     */
    void imgOpened(QString strImgPath);

    /** Signal for a zoom on the image
     *  @param dScale Scale of the image.
     */
    void zoomed(double dScale);

protected:

    /** Resize event.
     *  @param pEvent Resize event.
     */
    void resizeEvent(QResizeEvent* pEvent);

    /** Context menu event.
     *  @param pEvent ContextMenu event.
     */
    void contextMenuEvent(QContextMenuEvent* pEvent);

private:

    /** Center the image displaying in the window.
     */
    void centerImage();

    /** Resize the image displaying according with the given scale.
     *  @param dScale New scale.
     */
    void scaleImage(double dScale);

    //! Path of displayed image
    QString m_strImagePath;

    //! Label containing the image
    QLabel* m_pImageLabel;

    //! Scale factor (use for the image displaying)
    double m_dScaleFactor;

    //! Popup menu
    QMenu* m_pPopupMenu;

};

#endif

