/*
 * This source file is part of ImageViewer
 * For the latest info, see http://thierry.vouriot.free.fr
 * E-mail : vouriott@hotmail.com
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 * http://www.gnu.org/copyleft/lesser.txt.
 *
 */

#include "mainwindow.h"

#include <QDockWidget.h>
#include <QDirModel.h>
#include <QTreeView.h>
#include <QAction.h>
#include <QMenu.h>
#include <QMenuBar.h>
#include <QToolBar.h>
#include <QStatusBar.h>
#include <QMessageBox.h>

//=======================================================================
// MainWindow implementation
//=======================================================================

//-----------------------------------------------------------------------
MainWindow::MainWindow()
{
    m_pImageViewer = new ImageViewer(this);
    this->setCentralWidget(m_pImageViewer);

    m_pDock = new QDockWidget(tr("FileExplorer"), this);
    m_pDock->setAllowedAreas(Qt::LeftDockWidgetArea);

    m_pFileTreeExplorer = new FileTreeExplorer(m_pDock);
    m_pDock->setWidget(m_pFileTreeExplorer);

    this->addDockWidget(Qt::LeftDockWidgetArea, m_pDock);

    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

    m_pImageViewer->setPopupMenu(m_pViewMenu);

    setWindowTitle(tr("ImageViewer"));
    resize(640, 400);
    this->setWindowIcon(QIcon(":/icons/imageviewer.png"));

    connect(m_pFileTreeExplorer, SIGNAL(doubleClickedOnFile(QString)), m_pImageViewer, SLOT(openFile(QString)));
    connect(m_pImageViewer, SIGNAL(imgOpened(QString)), this, SLOT(open(QString)));
    connect(m_pImageViewer, SIGNAL(zoomed(double)), this, SLOT(zoom(double)));
}

//-----------------------------------------------------------------------
void MainWindow::createActions()
{
    m_pOpenAct = new QAction(QIcon(":/icons/open.png"), tr("&Open..."), this);
    m_pOpenAct->setShortcut(tr("Ctrl+O"));
    connect(m_pOpenAct, SIGNAL(triggered()), m_pImageViewer, SLOT(open()));

    m_pExitAct = new QAction(QIcon(":/icons/exit.png"), tr("E&xit"), this);
    m_pExitAct->setShortcut(tr("Ctrl+Q"));
    connect(m_pExitAct, SIGNAL(triggered()), this, SLOT(close()));

    m_pZoomInAct = new QAction(QIcon(":/icons/zoomin.png"), tr("Zoom &In (25%)"), this);
    m_pZoomInAct->setShortcut(tr("+"));
    m_pZoomInAct->setEnabled(false);
    connect(m_pZoomInAct, SIGNAL(triggered()), m_pImageViewer, SLOT(zoomIn()));

    m_pZoomOutAct = new QAction(QIcon(":/icons/zoomout.png"), tr("Zoom &Out (25%)"), this);
    m_pZoomOutAct->setShortcut(tr("-"));
    m_pZoomOutAct->setEnabled(false);
    connect(m_pZoomOutAct, SIGNAL(triggered()), m_pImageViewer, SLOT(zoomOut()));

    m_pNormalSizeAct = new QAction(QIcon(":/icons/normalscale.png"), tr("Normal &Size"), this);
    m_pNormalSizeAct->setShortcut(tr("Ctrl+S"));
    m_pNormalSizeAct->setEnabled(false);
    connect(m_pNormalSizeAct, SIGNAL(triggered()), m_pImageViewer, SLOT(normalSize()));

    m_pNextAct = new QAction(QIcon(":/icons/next.png"), tr("&Next Image"), this);
    m_pNextAct->setShortcut(tr("PgDown"));
    m_pNextAct->setEnabled(false);
    connect(m_pNextAct, SIGNAL(triggered()), this, SLOT(nextImage()));

    m_pPrevAct = new QAction(QIcon(":/icons/prev.png"), tr("&Previous Image"), this);
    m_pPrevAct->setShortcut(tr("PgUp"));
    m_pPrevAct->setEnabled(false);
    connect(m_pPrevAct, SIGNAL(triggered()), this, SLOT(prevImage()));

    m_pFullscreenAct = new QAction(QIcon(":/icons/fullscreen.png"), tr("&FullScreen"), this);
    m_pFullscreenAct->setShortcut(tr("F"));
    m_pFullscreenAct->setEnabled(false);
    connect(m_pFullscreenAct, SIGNAL(triggered()), this, SLOT(toggleFullscreen()));

    m_pAboutAct = new QAction(QIcon(":/icons/about.png"), tr("&About"), this);
    connect(m_pAboutAct, SIGNAL(triggered()), this, SLOT(about()));
}

//-----------------------------------------------------------------------
void MainWindow::createMenus()
{
    m_pFileMenu = new QMenu(tr("&File"), this);
    m_pFileMenu->addAction(m_pOpenAct);
    m_pFileMenu->addSeparator();
    m_pFileMenu->addAction(m_pExitAct);

    m_pViewMenu = new QMenu(tr("&View"), this);
    m_pViewMenu->addAction(m_pZoomInAct);
    m_pViewMenu->addAction(m_pZoomOutAct);
    m_pViewMenu->addAction(m_pNormalSizeAct);
    m_pViewMenu->addSeparator();
    m_pViewMenu->addAction(m_pPrevAct);
    m_pViewMenu->addAction(m_pNextAct);
    m_pViewMenu->addSeparator();
    m_pViewMenu->addAction(m_pFullscreenAct);

    m_pHelpMenu = new QMenu(tr("&Help"), this);
    m_pHelpMenu->addAction(m_pAboutAct);

    menuBar()->addMenu(m_pFileMenu);
    menuBar()->addMenu(m_pViewMenu);
    menuBar()->addMenu(m_pHelpMenu);
}

//-----------------------------------------------------------------------
void MainWindow::createToolBars()
{
    m_pToolBar = addToolBar(tr("toolbar"));

    m_pToolBar->addAction(m_pOpenAct);
    m_pToolBar->addSeparator();
    m_pToolBar->addAction(m_pZoomInAct);
    m_pToolBar->addAction(m_pZoomOutAct);
    m_pToolBar->addAction(m_pNormalSizeAct);
    m_pToolBar->addSeparator();
    m_pToolBar->addAction(m_pPrevAct);
    m_pToolBar->addAction(m_pNextAct);
    m_pToolBar->addAction(m_pFullscreenAct);
}

//-----------------------------------------------------------------------
void MainWindow::createStatusBar()
{
    statusBar()->showMessage(tr("Ready"));
}

//-----------------------------------------------------------------------
void MainWindow::open(QString strImgPath)
{
    m_pZoomInAct->setEnabled(true);
    m_pZoomOutAct->setEnabled(true);
    m_pNormalSizeAct->setEnabled(true);
    m_pNextAct->setEnabled(true);
    m_pPrevAct->setEnabled(true);

    m_pFullscreenAct->setEnabled(true);

    statusBar()->showMessage(strImgPath);
}

//-----------------------------------------------------------------------
void MainWindow::zoom(double dScale)
{
    if (dScale < 0.2) m_pZoomOutAct->setEnabled(false);
    else m_pZoomOutAct->setEnabled(true);

    if (dScale > 4.0) m_pZoomInAct->setEnabled(false);
    else m_pZoomInAct->setEnabled(true);
}

//-----------------------------------------------------------------------
void MainWindow::toggleFullscreen()
{
    if (isFullScreen())
    {
        showNormal();
        m_pDock->show();
        m_pToolBar->show();
        statusBar()->show();
        menuBar()->show();
    }
    else
    {
        m_pDock->hide();
        m_pToolBar->hide();
        statusBar()->hide();
        menuBar()->hide();
        showFullScreen();
    }
}

//-----------------------------------------------------------------------
void MainWindow::nextImage()
{
    changeImage(true);
}

//-----------------------------------------------------------------------
void MainWindow::prevImage()
{
    changeImage(false);
}

//-----------------------------------------------------------------------
void MainWindow::about()
{
    QMessageBox::about(this, tr("About ImageViewer"),
            tr("<p>Image viewer (JPG, BMP, PNG, GIF) with a file explorer developped by using the Qt library.</p>"
               "<p>Author: Thierry Vouriot <br />"
               "Email: <a href=\"mailto:vouriott@hotmail.com\">vouriott@hotmail.com</a> <br />"
               "Web: <a href=\"http://thierry.vouriot.free.fr\">http://thierry.vouriot.free.fr</a></p>"
               ));
}

//-----------------------------------------------------------------------
void MainWindow::changeImage(bool bNext)
{
    QDir dir(m_pImageViewer->getImagePath());
    dir.cdUp();
    dir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
    if (bNext) dir.setSorting(QDir::Name);
    else dir.setSorting(QDir::Name | QDir::Reversed);
    QString strFilter = "*.jpg *.jpeg *.bmp *.png *.gif";
    dir.setNameFilters(strFilter.split(' '));

    QFileInfoList list = dir.entryInfoList();

    // we search the index of the current image
    int nCurrentImageIdx = -1;
    for (int i = 0; i < list.size(); ++i) {
        QFileInfo fileInfo = list.at(i);
        if (fileInfo.filePath() == m_pImageViewer->getImagePath())
        {
            nCurrentImageIdx = i;
            break;
        }
    }

    // we open the next image
    if ((++nCurrentImageIdx) < list.size())
        m_pImageViewer->openFile(list.at(nCurrentImageIdx).filePath());
}





